##
## AF_INET6
##

package IO::Socket::INET6;

use strict;
use vars qw(@ISA $VERSION);
use Socket;
use Socket6;
use Carp;
use Exporter;
$VERSION="0.01a";

@ISA = qw(IO::Socket);

IO::Socket::INET6->register_domain( AF_INET6 );

my %socket_type = ( tcp => SOCK_STREAM,
	udp => SOCK_DGRAM,
	icmp => SOCK_RAW,
);

sub new
{
	my $class = shift;
	unshift(@_, "PeerAddr") if @_ == 1;
	return $class->SUPER::new(@_);
}

sub _sock_info {
	my ($addr,$port,$proto) = @_;
 
	my @proto = ();
	my @serv = ();
	if(defined $addr) {
		if ($addr =~ /^\[([0-9a-f:]+)\]\:(\d+)$/ ) {
			($port = $addr) =~ s/^\[([0-9a-f:]+)\]\:(\d+)$/$2/;
			$addr =~  s/^\[([0-9a-f:]+)\]\:(\d+)$/$1/;
		} elsif ($addr =~ /^(.+)\:(\d+)$/ ) {
			($port = $addr) =~ s/^(.+)\:(\d+)$/$2/;
			$addr =~  s/^(.+)\:(\d+)$/$1/;
		}
	}

	if(defined $proto) {
		@proto = $proto =~ m,\D, ? getprotobyname($proto): getprotobynumber($proto);
		$proto = $proto[2] || undef;
	}

	if(defined $port) {
		$port =~ s,\((\d+)\)$,,;

		my $defport = $1 || undef;
		my $pnum = ($port =~ m,^(\d+)$,)[0];

		@serv= getservbyname($port, $proto[0] || "") if($port =~ m,\D,);

		$port = $pnum || $serv[2] || $defport || undef;

		$proto = (getprotobyname($serv[3]))[2] || undef if @serv && !$proto;
	}
	
	return ($addr || undef, $port || undef, $proto || undef);
}

sub _error {
	  my $fh = shift;
	  $@ = join("",ref($fh),": ",@_);
	  carp $@ if $^W;
	  close($fh)
	if(defined fileno($fh));
	  return undef;
}

sub configure {
	my($fh,$arg) = @_;
	my($lport,$rport,$laddr,$raddr,$proto,$type);
	  
	($laddr,$lport,$proto) = _sock_info($arg->{LocalAddr},$arg->{LocalPort},$arg->{Proto});
	
	$laddr = inet_pton(AF_INET6,(defined $laddr?$laddr:"::"));
	
	return _error($fh,"Bad hostname '",$arg->{LocalAddr},"'") unless(defined $laddr);
	
	unless(exists $arg->{Listen}) {
		($raddr,$rport,$proto) = _sock_info($arg->{PeerAddr},$arg->{PeerPort}, $proto);
	}
	
	if(defined $raddr) {
		$raddr = inet_pton(AF_INET6,$raddr);	
		return _error($fh,"Bad hostname '",$arg->{PeerAddr},"'") unless(defined $raddr);
	}

	
	$proto ||= (getprotobyname "tcp")[2];
	return _error($fh,'Cannot determine protocol')	unless($proto);
	
	my $pname = (getprotobynumber($proto))[0];
	$type = $arg->{Type} || $socket_type{$pname};
	
	$fh->socket(AF_INET6, $type, $proto) or return _error($fh,"$!");

	if ($arg->{Reuse}) {
		$fh->sockopt(SO_REUSEADDR,1) or return _error($fh);
	}

	$fh->bind(pack_sockaddr_in6($lport || 0, $laddr)) or return _error($fh,"$!");
	
	if(exists $arg->{Listen}) {
		$fh->listen($arg->{Listen} || 5) or return _error($fh,"$!");
	} else {
		return _error($fh,'Cannot determine remote port') unless($rport || $type == SOCK_DGRAM || $type == SOCK_RAW);

		if($type == SOCK_STREAM || defined $raddr) {
			return _error($fh,'Bad peer address') unless(defined $raddr);
			$fh->connect(pack_sockaddr_in6($rport ,$raddr)) or return _error($fh,"$!");
		}
	}
	$fh;
}

sub sockaddr {
	@_ == 1 or croak 'usage: $fh->sockaddr()';
	my($fh) = @_;
	(unpack_sockaddr_in6($fh->sockname))[1];
}

sub sockport {
	@_ == 1 or croak 'usage: $fh->sockport()';
	my($fh) = @_;
	(unpack_sockaddr_in6($fh->sockname))[0];
}

sub sockhost {
	@_ == 1 or croak 'usage: $fh->sockhost()';
	my($fh) = @_;
	inet_ntop(AF_INET6,$fh->sockaddr);
}

sub peeraddr {
	@_ == 1 or croak 'usage: $fh->peeraddr()';
	my($fh) = @_;
	(unpack_sockaddr_in6($fh->peername))[1];
}

sub peerport {
	@_ == 1 or croak 'usage: $fh->peerport()';
	my($fh) = @_;
	(unpack_sockaddr_in6($fh->peername))[0];
}

sub peerhost {
	@_ == 1 or croak 'usage: $fh->peerhost()';
	my($fh) = @_;
	inet_ntop(AF_INET6,$fh->peeraddr);
}



